@@ -4,6 +4,8 @@ import pathlib
 import base64
 from copy import copy
 import graphene
+from insuree.services import validate_insuree_number
+
 from .apps import InsureeConfig
 from core import filter_validity, assert_string_length
 from core.schema import TinyInt, SmallInt, OpenIMISMutation
@@ -83,6 +85,8 @@ class FamilyBase:
     confirmation_type_id = graphene.String(max_length=3, required=False)
     json_ext = graphene.types.json.JSONString(required=False)
 
+    contribution = graphene.types.json.JSONString(required=False)
+
     head_insuree = graphene.Field(FamilyHeadInsureeInputType, required=False)
 
 
@@ -160,7 +164,7 @@ def handle_insuree_photo(user, now, insuree, data):
     photo_bin = data.get('photo', None)
     if photo_bin and InsureeConfig.insuree_photos_root_path and photo_bin != insuree_photo.photo:
         (file_dir, file_name) = create_file(now, insuree.id, photo_bin)
-        data.pop('photo')
+        data.pop('photo', None)
         data['folder'] = file_dir
         data['filename'] = file_name
 
@@ -179,16 +183,14 @@ def handle_insuree_photo(user, now, insuree, data):
 
 
 def update_or_create_insuree(data, user):
-    if "client_mutation_id" in data:
-        data.pop('client_mutation_id')
-    if "client_mutation_label" in data:
-        data.pop('client_mutation_label')
+    data.pop('client_mutation_id', None)
+    data.pop('client_mutation_label', None)
     photo = data.pop('photo', None)
     from core import datetime
     now = datetime.datetime.now()
     data['audit_user_id'] = user.id_for_audit
     data['validity_from'] = now
-    insuree_uuid = data.pop('uuid') if 'uuid' in data else None
+    insuree_uuid = data.pop('uuid', None)
     # update_or_create(uuid=insuree_uuid, ...)
     # doesn't work because of explicit attempt to set null to uuid!
     if insuree_uuid:
@@ -199,7 +201,11 @@ def update_or_create_insuree(data, user):
         reset_insuree_before_update(insuree)
         [setattr(insuree, key, data[key]) for key in data]
     else:
-        insuree = Insuree.objects.create(**data)
+        errors = validate_insuree_number(data["chf_id"])
+        if errors:
+            raise Exception("Invalid insuree number")
+        else:
+            insuree = Insuree.objects.create(**data)
     insuree.save()
     photo = handle_insuree_photo(user, now, insuree, photo)
     if photo:
@@ -222,15 +228,13 @@ def reset_family_before_update(family):
 
 
 def update_or_create_family(data, user):
-    if "client_mutation_id" in data:
-        data.pop('client_mutation_id')
-    if "client_mutation_label" in data:
-        data.pop('client_mutation_label')
+    data.pop('client_mutation_id', None)
+    data.pop('client_mutation_label', None)
     head_insuree_data = data.pop('head_insuree')
     head_insuree_data["head"] = True
     head_insuree = update_or_create_insuree(head_insuree_data, user)
     data["head_insuree"] = head_insuree
-    family_uuid = data.pop('uuid') if 'uuid' in data else None
+    family_uuid = data.pop('uuid', None)
     # update_or_create(uuid=family_uuid, ...)
     # doesn't work because of explicit attempt to set null to uuid!
     if family_uuid:
@@ -241,6 +245,7 @@ def update_or_create_family(data, user):
         reset_family_before_update(family)
         [setattr(family, key, data[key]) for key in data]
     else:
+        data.pop('contribution', None)
         family = Family.objects.create(**data)
     family.save()
     head_insuree.family = family
@@ -270,10 +275,15 @@ class CreateFamilyMutation(OpenIMISMutation):
             from core.utils import TimeUtils
             data['validity_from'] = TimeUtils.now()
             client_mutation_id = data.get("client_mutation_id")
+            # Validate insuree number right away
+            errors = validate_insuree_number(data.get("head_insuree", {}).get("chf_id", None))
+            if errors:
+                return errors
             family = update_or_create_family(data, user)
             FamilyMutation.object_mutated(user, client_mutation_id=client_mutation_id, family=family)
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_create_family")
             return [{
                 'message': _("insuree.mutation.failed_to_create_family"),
                 'detail': str(exc)}
@@ -304,8 +314,9 @@ class UpdateFamilyMutation(OpenIMISMutation):
             FamilyMutation.object_mutated(user, client_mutation_id=client_mutation_id, family=family)
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_update_family")
             return [{
-                'message': _("insuree.mutation.failed_to_create_family"),
+                'message': _("insuree.mutation.failed_to_update_family"),
                 'detail': str(exc)}
             ]
 
@@ -326,6 +337,7 @@ def set_family_deleted(family, delete_members):
         family.delete_history()
         return []
     except Exception as exc:
+        logger.exception("insuree.mutation.failed_to_delete_family")
         return {
             'title': family.uuid,
             'list': [{
@@ -389,10 +401,15 @@ class CreateInsureeMutation(OpenIMISMutation):
             from core.utils import TimeUtils
             data['validity_from'] = TimeUtils.now()
             client_mutation_id = data.get("client_mutation_id")
+            # Validate insuree number right away
+            errors = validate_insuree_number(data.get("chf_id", None))
+            if errors:
+                return errors
             insuree = update_or_create_insuree(data, user)
             InsureeMutation.object_mutated(user, client_mutation_id=client_mutation_id, insuree=insuree)
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_create_insuree")
             return [{
                 'message': _("insuree.mutation.failed_to_create_insuree"),
                 'detail': str(exc)}
@@ -423,6 +440,7 @@ class UpdateInsureeMutation(OpenIMISMutation):
             InsureeMutation.object_mutated(user, client_mutation_id=client_mutation_id, insuree=insuree)
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_update_insuree")
             return [{
                 'message': _("insuree.mutation.failed_to_update_insuree"),
                 'detail': str(exc)}
@@ -435,6 +453,7 @@ def set_insuree_deleted(insuree):
         [ip.delete_history() for ip in insuree.insuree_policies.filter(validity_to__isnull=True)]
         return []
     except Exception as exc:
+        logger.exception("insuree.mutation.failed_to_delete_insuree")
         return {
             'title': insuree.chf_id,
             'list': [{
@@ -494,6 +513,7 @@ def cancel_policies(insuree):
         InsureePolicy.objects.bulk_update(ips, ['expiry_date'])
         return []
     except Exception as exc:
+        logger.exception("insuree.mutation.failed_to_cancel_insuree_policies")
         return {
             'title': insuree.chf_id,
             'list': [{
@@ -509,6 +529,7 @@ def remove_insuree(insuree):
         insuree.save()
         return []
     except Exception as exc:
+        logger.exception("insuree.mutation.failed_to_remove_insuree")
         return {
             'title': insuree.chf_id,
             'list': [{
@@ -592,6 +613,7 @@ class SetFamilyHeadMutation(OpenIMISMutation):
             insuree.save()
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_set_head_insuree")
             return [{
                 'message': _("insuree.mutation.failed_to_set_head_insuree"),
                 'detail': str(exc)}
@@ -625,6 +647,7 @@ class ChangeInsureeFamilyMutation(OpenIMISMutation):
                 return cancel_policies(insuree)
             return None
         except Exception as exc:
+            logger.exception("insuree.mutation.failed_to_change_insuree_family")
             return [{
                 'message': _("insuree.mutation.failed_to_change_insuree_family"),
                 'detail': str(exc)}
